package com.brilliance.swift.mx2mt;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.TemplateUtil;
import com.brilliance.swift.util.XmlUtil;
import com.prowidesoftware.swift.model.mx.AbstractMX;
import com.prowidesoftware.swift.model.mx.AppHdr;
import com.prowidesoftware.swift.model.mx.BusinessAppHdrV01;
import com.prowidesoftware.swift.model.mx.BusinessAppHdrV02;
import com.prowidesoftware.swift.model.mx.dic.Priority2Code;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;

public abstract class AbstractMx2MtCreator implements Mx2MtCreator {

    private static final Logger logger = LoggerFactory.getLogger(AbstractMx2MtCreator.class);

    protected Map<String, Object> map = new HashMap<>();

    protected AbstractMx2MtCreator withXmlDocument(String xmlStr) {
        try {
            Document document = DocumentHelper.parseText(xmlStr);
            map.put(Mx2MtConstants.MX_XML_DOCUMENT, document);
            return this;
        }catch (DocumentException e) {
            throw new SwiftException(e.getMessage());
        }
    }

    protected AbstractMx2MtCreator withObj(Object obj) {
        map.put(Mx2MtConstants.MX_OBJECT, obj);
        return this;
    }

    protected AbstractMx2MtCreator withOutPutFilePath(String outPutFilePath) {
        if (!StringUtil.isEmpty(outPutFilePath)) {
             map.put(Mx2MtConstants.OUT_PUT_FILE_PATH, outPutFilePath);
        }
        return this;
    }

    protected AbstractMx2MtCreator withExtraMap(Map<String, Object> extraMap) {
        if (extraMap != null && extraMap.size() > 0) {
            extraMap.forEach((key, value) -> {
                map.put(key, value);
            });
        }
        if (!map.containsKey(Mx2MtConstants.MT_IO_TYPE)) {
            map.put(Mx2MtConstants.MT_IO_TYPE, Mx2MtConstants.MT_IO_TYPE_OUTPUT); //default value is O
        }
        return this;
    }

    public Map<String, Object> getMap() {
        return map;
    }

    public void setMap(Map<String, Object> map) {
        this.map = map;
    }

    //TODO
    protected String getRandomNumber(int length) {
        String randomNumber = "";
        Random random = new Random();
        for (int i=0; i<length; i++) {
            randomNumber += random.nextInt(9);
        }
        return randomNumber;
    }

    protected String getPriority(AppHdr appHdr) {
        String priority = Mx2MtConstants.HEADER_PRIORITY_NORMAL;
        if (appHdr != null) {
            if (appHdr instanceof BusinessAppHdrV01) {
                BusinessAppHdrV01 businessAppHdrV01 = (BusinessAppHdrV01)appHdr;
                if (StringUtil.isNotEmpty(businessAppHdrV01.getPrty())) {
                    if (Priority2Code.HIGH.equals(Priority2Code.fromValue(businessAppHdrV01.getPrty()))) {
                        priority = Mx2MtConstants.HEADER_PRIORITY_HIGH;
                    }
                }
            } else if (appHdr instanceof BusinessAppHdrV02) {
                BusinessAppHdrV02 businessAppHdrV02 = (BusinessAppHdrV02)appHdr;
                if (StringUtil.isNotEmpty(businessAppHdrV02.getPrty())) {
                    if (Priority2Code.HIGH.equals(Priority2Code.fromValue(businessAppHdrV02.getPrty()))) {
                        priority = Mx2MtConstants.HEADER_PRIORITY_HIGH;
                    }
                }
            }
        }
        return priority;
    }

    protected String getXmlNodeValue(String type, Document document, String path) {
        String parentPath = (String)map.get(type);
        if (StringUtil.isNotEmpty(parentPath)) {
            path = parentPath + "." + path;
        }
        return XmlUtil.getXmlNodeValue(document, path);
    }

    protected String processBicCode(String bicCode) {
        String processedBicCode = "";
        if (!StringUtil.isEmpty(bicCode)) {
            if (bicCode.length() == 8) {
                processedBicCode = bicCode + Mx2MtConstants.BICSUFFIX;
            } else if (bicCode.length() == 11) {
                processedBicCode = bicCode.substring(0, 8) + Mx2MtConstants.BICMIDDLE + bicCode.substring(8);
            } else {
                throw new SwiftException("ERROR", "BicCode length is invalid.");
            }
        }
        return processedBicCode;
    }

    protected abstract String getMtType();

    @Override
    public void preProcess() {

    }
    /**
     * process method
     */
    @Override
    public void generateHead1() {
        String headApplicationIden = Mx2MtConstants.HEADER_APPLICATION_IDEN_F;
        String headServiceType = Mx2MtConstants.HEADER_SERVICE_TYPE_01;
        String bicCode = "";
        String sessionNumber = getRandomNumber(4);
        String sequenceNumber = getRandomNumber(6);
        String mtIOType = (String)map.get(Mx2MtConstants.MT_IO_TYPE);
        AbstractMX abstractMX = (AbstractMX)map.get(Mx2MtConstants.MX_OBJECT);
        AppHdr appHdr = abstractMX.getAppHdr();
        if (appHdr != null) {
            if (Mx2MtConstants.MT_IO_TYPE_INPUT.equals(mtIOType)) {
                bicCode = processBicCode(appHdr.from());
            } else if (Mx2MtConstants.MT_IO_TYPE_OUTPUT.equals(mtIOType)) {
                bicCode = processBicCode(appHdr.to());
            }
        }
        String head1 = headApplicationIden + headServiceType + bicCode + sessionNumber + sequenceNumber;
        map.put("head1", head1);
    }

    @Override
    public void generateHead2() {
        String mtIOType = (String)map.get(Mx2MtConstants.MT_IO_TYPE);
        String mtType = getMtType();
        AbstractMX abstractMX = (AbstractMX)map.get(Mx2MtConstants.MX_OBJECT);
        AppHdr appHdr = abstractMX.getAppHdr();
        String priority  = getPriority(appHdr);
        String head2 = "";
        if (Mx2MtConstants.MT_IO_TYPE_INPUT.equals(mtIOType)) {
            String bicCode = "";
            if (appHdr != null) {
                bicCode = processBicCode(appHdr.to());
            }
            head2 = mtIOType + mtType + bicCode + priority;
        } else if (Mx2MtConstants.MT_IO_TYPE_OUTPUT.equals(mtIOType)) {
            String createDateStr = "";
            String createTimeStr = "";
            String outDateStr = DateUtil.formatNow( "yyMMdd");
            String outTimeStr = DateUtil.formatNow("HHmm");
            String bicCode = "";
            String sessionNumber = getRandomNumber(4);
            String sequenceNumber = getRandomNumber(6);
            if (appHdr != null) {
                bicCode = processBicCode(appHdr.from());
                if (appHdr.creationDate() != null) {
                    createDateStr = DateUtil.format(appHdr.creationDate(), "yyMMdd");
                    createTimeStr = DateUtil.format(appHdr.creationDate(), "HHmm");
                } else {
                    createDateStr = DateUtil.formatNow("yyMMdd");
                    createTimeStr = DateUtil.formatNow("HHmm");
                }
            }
            StringBuilder sb = new StringBuilder();
            sb.append(mtIOType)
              .append(mtType)
              .append(createTimeStr)
              .append(createDateStr)
              .append(bicCode)
              .append(sessionNumber)
              .append(sequenceNumber)
              .append(outTimeStr)
              .append(outDateStr)
              .append(priority);
            head2 = sb.toString();
        }
        map.put("head2", head2);
    }

    @Override
    public void generateHead3() {
        //need childClasses implements
        map.put("head3", "");
    }

    @Override
    public void generateHead5() {
        map.put("head5", "");
    }

    @Override
    public void postProcess() {

    }

    public String generateMtMsg() {
        String outPutFilePath = (String)map.get(Mx2MtConstants.OUT_PUT_FILE_PATH);
        String mtTemplateName = (String)map.get(Mx2MtConstants.MT_TEMPLATE_NAME);
        String msg = TemplateUtil.makeFileByTemplate(mtTemplateName, map, outPutFilePath, "utf-8");
        return msg;
    }
}
