package com.brilliance.swift.mx2mt;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.mt103.Mx2Mt103Creator;
import com.brilliance.swift.mx2mt.mt202cov202.Mx2Mt202Creator;
import com.brilliance.swift.mx2mt.mt900910.Mx2Mt900910Creator;
import com.brilliance.swift.mx2mt.mt940950.Mx2Mt940950Creator;
import com.prowidesoftware.swift.model.mx.AbstractMX;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.List;
import java.util.Map;

public class Mx2MtCreateManager {

    private static final Logger logger = LoggerFactory.getLogger(Mx2MtCreateManager.class);

    private String msgType = "";

    private Object mxObj = null;

    private String xmlStr = "";

    public String getMsgType() {
        return msgType;
    }

    public void setMsgType(String msgType) {
        this.msgType = msgType;
    }

    public Object getMxObj() {
        return mxObj;
    }

    public void setMxObj(Object mxObj) {
        this.mxObj = mxObj;
    }

    public void init(String xmlStr) {
        AbstractMX abstractMX = AbstractMX.parse(xmlStr);
        msgType = (abstractMX.getMxId().getBusinessProcess().name()
                + abstractMX.getMxId().getFunctionality()
                + abstractMX.getMxId().getVariant())
                //+ abstractMX.getMxId().getVersion())
                .trim()
                .replace(".", "");
        mxObj = abstractMX;
        this.xmlStr = xmlStr;
    }

    public String mx2mt(File file, String outPutFilePath, Map<String, Object> extraMap) throws Exception {
        String xmlStr = FileUtils.readFileToString(file);
        init(xmlStr);
        return convert(outPutFilePath, extraMap);
    }

    public String mx2mt(File file, String outPutFilePath) throws Exception {
        return mx2mt(file, outPutFilePath, null);
    }

    public String mx2mt(File file) throws Exception {
        return mx2mt(file, null, null);
    }

    public String mx2mt(String xmlStr, String outPutFilePath, Map<String, Object> extraMap) throws Exception {
        init(xmlStr);
        return convert(outPutFilePath, extraMap);
    }

    public String mx2mt(String xmlStr, String outPutFilePath) throws Exception {
        return mx2mt(xmlStr, outPutFilePath, null);
    }

    public String mx2mt(String xmlStr) throws Exception {
        return mx2mt(xmlStr, null, null);
    }

    public String convert(String outPutFilePath, Map<String, Object> extraMap) throws SwiftException {
        AbstractMx2MtCreator creator = getMx2MtCreator(msgType);
        creator.withXmlDocument(xmlStr).withObj(mxObj).withOutPutFilePath(outPutFilePath).withExtraMap(extraMap);
        creator.preProcess();
        creator.generateHead1();
        creator.generateHead2();
        creator.generateHead3();
        List<Mx2MtFieldsGenerate> generateList = creator.getFieldsGenerateList();
        for (int i=0; i<generateList.size(); i++) {
            AbstractMx2MtFieldsGenerate fieldsGenerate = (AbstractMx2MtFieldsGenerate) generateList.get(i);
            fieldsGenerate.withMap(creator.getMap());
            fieldsGenerate.fieldsGenerate();
        }
        creator.generateHead5();
        creator.postProcess();
        return creator.generateMtMsg();
    }

    public AbstractMx2MtCreator getMx2MtCreator(String msgType) throws SwiftException {
        if ("pacs008001".equals(msgType)) {
            return new Mx2Mt103Creator();
        } else if ("pacs009001".equals(msgType)){
            return new Mx2Mt202Creator();
        } else if ("camt053001".equals(msgType)){
            return new Mx2Mt940950Creator();
        } else if ("camt054001".equals(msgType)){
            return new Mx2Mt900910Creator();
        } else {
            throw new SwiftException("Invalid message type");
        }
    }
}
