package com.brilliance.swift.mx2mt.mt103.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * //1）code部分的取值：
 *         //1.1）若<PmtTpInf><SvcLvl><Cd>节点为“SDVA”，则取值“SDVA”；
 *         //1.2）若<CtgyPurp><Cd>节点有值，则直接取此节点作为23E的值；
 *         //1.3）若<InstrForCdtrAgt><Cd>有值，则直接取此节点作为23E的值
 *         //2）直接取节点<InstrInf>的内容作为addtion information的值
 *         <CdtTrfTxInf>[1]<PmtTpInf><CtgyPurp><Cd>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><Cd>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><InstrInf>
 */
public class Field23EGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field23EGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            List<String> field23EList = new ArrayList<>();
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String svclvlCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PmtTpInf.SvcLvl.Cd");
            if ("SDVA".equals(svclvlCd)) {
                field23EList.add(svclvlCd);
            }
            String ctgyPurpCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PmtTpInf.CtgyPurp.Cd");
            if (StringUtil.isNotEmpty(ctgyPurpCd)) {
                field23EList.add(ctgyPurpCd);
            }
            String instrForCdtrAgtCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.InstrForCdtrAgt.Cd");
            if (StringUtil.isNotEmpty(instrForCdtrAgtCd)) {
                String instrInf = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.InstrForCdtrAgt.InstrInf");
                if (StringUtil.isNotEmpty(instrInf)) {
                    if (instrInf.length() > 30) {
                        instrInf = instrInf.substring(0, 30);
                    }
                    instrForCdtrAgtCd += "/" + instrInf;
                }
                field23EList.add(instrForCdtrAgtCd);
            }
            map.put("field23EList", field23EList);
        }  catch(Exception e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
