package com.brilliance.swift.mx2mt.mt103.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * //1,若节点<BICFI>有值，则设置56 option为“A”，同时BIC取此栏位；
 *         //2,若节点<IBAN>有值，则作为56A的账号；
 *         //3,若节点<Othr><Id>有值，则作为56A的账号；
 *         //<CdtTrfTxInf>[1]<IntrmyAgt1><FinInstnId><BICFI>
 *         //<CdtTrfTxInf>[1]<IntrmyAgt1Acct><Id><IBAN>
 *         //<CdtTrfTxInf>[1]<IntrmyAgt1Acct><Id><Othr><Id>
 *         //<CdtTrfTxInf>[1]<IntrmyAgt1Acct><Id><Othr><SchmeNm><Cd>
 *              //如果<IntrmyAgt1><FinInstnId><BICFI>为空，且<IntrmyAgt1><FinInstnId><Nm>为空，
 *  *         //但节点<IBAN>有值，或者节点<Othr><Id>有值，则设置56option为“C”
 *  *         //若节点<Id><Othr><Id>不为空，则取“//CH”+此节点作为账号，否则取节点<MmbId>作为账号：
 *  *         //<CdtTrfTxInf>[1]<IntrmyAgt1><FinInstnId><ClrSysMmbId><ClrSysId><Cd>
 *  *         //<CdtTrfTxInf>[1]<IntrmyAgt1><FinInstnId><ClrSysMmbId><ClrSysId><Prtry>
 *  *         //<CdtTrfTxInf>[1]<IntrmyAgt1><FinInstnId><ClrSysMmbId><MmbId>
 *  *         //<CdtTrfTxInf>[1]<IntrmyAgt1Acct><Id><Othr><Id>
 *       *         //1，若节点<Nm>有值，则设置56 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  *         //2，账号取值规则和节点与56A相同
 *  *         //<CdtTrfTxInf>[1]<IntrmyAgt1><FinInstnId><Nm>
 *  *         //<CdtTrfTxInf>[1]<IntrmyAgt1><FinInstnId><PstlAdr><AdrLine>[1],<AdrLine>[2],<AdrLine>[3]
 */
public class Field56AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field56AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field56Option = "";
            String field56 = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1.FinInstnId");
            String account = getAccount();
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1.FinInstnId.Nm");
            if (StringUtil.isNotEmpty(bicCode)) {
                field56Option = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field56 = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field56 = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)){
                field56Option = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field56 = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field56 = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field56 += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            } else if (StringUtil.isNotEmpty(account)){
                field56Option = "C";
                String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1Acct.Id.IBAN");
                String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1Acct.Id.Othr.Id");
                String mmbId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1.FinInstnId.ClrSysMmbId.MmbId");
                //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
                if (StringUtil.isNotEmpty(otherId)) {
                    account = "//CH" + otherId;
                } else if (StringUtil.isNotEmpty(mmbId)) {
                    account = "/" + mmbId;
                } else {
                    account = "/" + iban;
                }
                if (account.length() > 35) {
                    account = account.substring(0, 35);
                }
                field56 = account;
            }
            map.put("field56Option", field56Option);
            map.put("field56", field56);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1Acct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.IntrmyAgt1Acct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}
