package com.brilliance.swift.mx2mt.mt103.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.NumberUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.mx.dic.ChargeBearerType1Code;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;

/**
 * 1）如果节点值为 'CRED'，则71A取值为“BEN”
 * 2）如果节点值为 'DEBT'，则71A取值为“OUR”
 * 3）如果节点值为'SHAR'，则71A取值为“SHA”
 * <CdtTrfTxInf>[1]<ChrgBr>
 *      * //1，若71A为“BEN”或“SHA”，则此节点取值为71F
 *  *         //CdtTrfTxInf>[1]<ChrgsInf><Amt Ccy="XXX">
 *       * 1，若71A为“OUR”，则此节点取值为71G；
 *  * <CdtTrfTxInf>[1]<ChrgsInf><Amt Ccy="XXX">
 */
public class Field71AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field71AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field71Option = "";
            String field71 = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String chrgBr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.ChrgBr");
            if (StringUtil.isNotEmpty(chrgBr)) {
                if (ChargeBearerType1Code.CRED.value().equals(chrgBr)) {
                    field71Option = "BEN";
                } else if (ChargeBearerType1Code.DEBT.value().equals(chrgBr)) {
                    field71Option = "OUR";
                } else if (ChargeBearerType1Code.SHAR.value().equals(chrgBr)) {
                    field71Option = "SHA";
                }
                String amt = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.ChrgsInf");
                String ccy = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.ChrgsInf@Ccy");
                if (StringUtil.isNotEmpty(amt) && StringUtil.isNotEmpty(ccy)) {
                    field71 = ccy + NumberUtil.formatAmt(new BigDecimal(amt), ccy);
                }
            }
            map.put("field71Option", field71Option);
            map.put("field71", field71);
        } catch(Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}
