package com.brilliance.swift.mx2mt.mt202cov202;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.mx2mt.AbstractMx2MtCreator;
import com.brilliance.swift.mx2mt.Mx2MtFieldsGenerate;
import com.brilliance.swift.mx2mt.mt202cov202.impl.*;
import com.brilliance.swift.mx2mt.mt202cov202.impl.cov.*;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class Mx2Mt202Creator extends AbstractMx2MtCreator {
    private static final Logger logger = LoggerFactory.getLogger(Mx2Mt202Creator.class);

    @Override
    public void preProcess() {
        Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
        Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
        map.put(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, parentElementMaps.get("GrpHdr"));
        map.put(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, parentElementMaps.get("CdtTrfTxInf"));
        map.put(Mx2MtConstants.MT_TEMPLATE_NAME, "swift_mt_202cov.vm");
    }

    @Override
    public void generateHead3() {
        String head3 = "";
        Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
        String uetr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PmtId.UETR");
        if (StringUtil.isNotEmpty(uetr)) {
            head3 += "{121:"+uetr+"}";
        }
        boolean mt202CoverFlag = false;
        Map<String, String> maps = XmlUtil.getParentElementMaps(document);
        Iterator<String> keys = maps.keySet().iterator();
        for (; keys.hasNext(); ) {
            String key = keys.next();
            String value = maps.get(key);
            String fullPath = value + "." + key;
            if (fullPath.indexOf("UndrlygCstmrCdtTrf") > -1
                    && StringUtil.isNotEmpty(XmlUtil.getXmlNodeValue(document, fullPath))){
                mt202CoverFlag = true;
                break;
            }
        }
        if (mt202CoverFlag) {
            head3 += Mx2MtConstants.HEAD3_MT202_COVER;
        }
        map.put("head3", head3);
    }
    @Override
    public List<Mx2MtFieldsGenerate> getFieldsGenerateList() {
        List<Mx2MtFieldsGenerate> fieldsGenerateList = new ArrayList<>();
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field21Generate());
        fieldsGenerateList.add(new Field13CGenerate());
        fieldsGenerateList.add(new Field32AGenerate());
        fieldsGenerateList.add(new Field52AGenerate());
        fieldsGenerateList.add(new Field53AGenerate());
        fieldsGenerateList.add(new Field54AGenerate());
        fieldsGenerateList.add(new Field56AGenerate());
        fieldsGenerateList.add(new Field57AGenerate());
        fieldsGenerateList.add(new Field58AGenerate());
        fieldsGenerateList.add(new Field72Generate());
        fieldsGenerateList.add(new Field50AGenerate());
        fieldsGenerateList.add(new FieldB52AGenerate());
        fieldsGenerateList.add(new FieldB56Generate());
        fieldsGenerateList.add(new FieldB57Generate());
        fieldsGenerateList.add(new FieldB59Generate());
        fieldsGenerateList.add(new FieldB70Generate());
        fieldsGenerateList.add(new FieldB72Generate());
        fieldsGenerateList.add(new FieldB33BGenerate());
        return fieldsGenerateList;
    }

    @Override
    public void postProcess() {

    }

    @Override
    protected String getMtType() {
        return Mx2MtConstants.MT_TYPE_202;
    }

}