package com.brilliance.swift.mx2mt.mt202cov202.impl;


import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 1,若节点<BICFI>有值，则设置53 option为“A”，同时BIC取此栏位；
 * 2,若节点<IBAN>有值，则作为53A的账号；
 * 3,若节点<Othr><Id>有值，则作为53A的账号；
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgt><FinInstnId><BICFI>
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgtAcct><Id><IBAN>
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgtAcct><Id><Othr><Id>
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgtAcct><Id><Othr><SchmeNm><Cd> * 如果<SttlmAcct><Id><IBAN>或<SttlmAcct><Id><Othr><Id>有值，则设置53 option为“B”：
 *  * 1）若节点<IBAN>有值，则作为53B的账号；
 *  * 2）若节点<Othr><Id>有值，则作为53B的账号；
 *  * <GrpHdr><SttlmInf><SttlmAcct><Id><IBAN>
 *  * <GrpHdr><SttlmInf><SttlmAcct><Id><Othr><Id>
 *  * <GrpHdr><SttlmInf><SttlmAcct><Id><Othr><SchmeNm><Cd>
 *       * 1,若节点<Nm>有值，则设置53 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2,账号取值规则和节点与53A相同
 *  * <GrpHdr><SttlmInf><InstgRmbrsmntAgt><FinInstnId><Nm>
 *  * <GrpHdr><SttlmInf><InstgRmbrsmntAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class Field53AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field53AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field53Option = "";
            String field53 = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.InstgRmbrsmntAgt.FinInstnId");
            String account = getAccount();
            String sttlmAccount = getSttlmAccount();
            String name = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.InstgRmbrsmntAgt.FinInstnId.Nm");
            if (StringUtil.isNotEmpty(bicCode)) {
                field53Option = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field53 = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field53 = bicCode;
                }
            } else if (StringUtil.isNotEmpty(sttlmAccount)){
                field53Option = "B";
                field53 = sttlmAccount;
            } else if (StringUtil.isNotEmpty(name)){
                field53Option = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field53 = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field53 = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.InstgRmbrsmntAgt.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.InstgRmbrsmntAgt.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field53 += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            }
            map.put("field53Option", field53Option);
            map.put("field53A", field53);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.InstgRmbrsmntAgtAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.InstgRmbrsmntAgtAcct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getSttlmAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.SttlmAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, document, "GrpHdr.SttlmInf.SttlmAcct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}

