package com.brilliance.swift.mx2mt.mt202cov202.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 1,若节点<BICFI>有值，则设置58 option为“A”，同时BIC取此栏位；
 * 2,若节点<IBAN>有值，则作为58A的账号；
 * 3,若节点<Othr><Id>有值，则作为58A的账号；
 * <CdtTrfTxInf>[1]<Cdtr><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<CdtrAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<CdtrAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<CdtrAcct><Id><Othr><SchmeNm><Cd>
 *      * 1,若节点<Nm>有值，则设置58 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2,账号取值规则和节点与58A相同
 *  * <CdtTrfTxInf>[1]<Cdtr><FinInstnId><Nm>
 *  * <CdtTrfTxInf>[1]<Cdtr><FinInstnId><PstlAdr><AdrLine>[1],<AdrLine>[2],<AdrLine>[3]
 */
public class Field58AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field58AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field58Option = "";
            String field58 = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.Cdtr.FinInstnId");
            String account = getAccount();
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.Cdtr.FinInstnId.Nm");
            if (StringUtil.isNotEmpty(bicCode)) {
                field58Option = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field58 = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field58 = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)){
                field58Option = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field58 = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field58 = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.Cdtr.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.Cdtr.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field58 += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            }
            map.put("field58Option", field58Option);
            map.put("field58A", field58);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.CdtrAcct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}

