package com.brilliance.swift.mx2mt.mt202cov202.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.mx.dic.Instruction4Code;
import com.prowidesoftware.swift.model.mx.dic.Instruction5Code;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 1，若<BICFI>有值，则转为“/INS/此节点的值”
 * 2，如果<Nm>有值，则取值为“/INS/”+此节点的值，超过35字节则将剩下内容用“//内容“的方式进行截取
 * 3，如果节点<InstrForCdtrAgt><Cd>没有值，但节点<InstrForCdtrAgt><InstrInf>有值，则取值“/ACC/”+<InstrForCdtrAgt><InstrInf>节点的值
 * 4，如果节点<InstrForCdtrAgt><Cd>值为PHOB，则取值“/PHONBEN/”+<InstrForCdtrAgt><InstrInf>节点的值
 * 5，如果节点<InstrForCdtrAgt><Cd>值为TELB，则取值“/PHONBEN/”+<InstrForCdtrAgt><InstrInf>节点的值
 * 6，如果节点<InstrForNxtAgt><Cd>值为PHOA，且56a不存在，则取值“/PHON/”+<InstrForNxtAgt><InstrInf>节点的值
 * 7，如果节点<InstrForNxtAgt><Cd>值为TELA，则取值“/TELE/”+<InstrForNxtAgt><InstrInf>节点的值
 * 8，如果节点<InstrForNxtAgt><Cd>值为空，且<InstrInf>节点有值，则取值“/REC/”+<InstrForNxtAgt><InstrInf>节点的值
 * 9，如果节点<InstrForNxtAgt><Cd>值为PHOA，且56a存在，则取值“/PHONIBK/”+<InstrForNxtAgt><InstrInf>节点的值
 * <CdtTrfTxInf>[1]<PrvsInstgAgt1><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<PrvsInstgAgt1><FinInstnId><Nm>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><Cd>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><InstrInf>
 * <CdtTrfTxInf>[1]<InstrForNxtAgt><Cd>
 * <CdtTrfTxInf>[1]<InstrForNxtAgt><InstrInf>
 * <CdtTrfTxInf>[1]<RmtInf><Ustrd>
 */
public class Field72Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field72Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            List<String> field72List = new ArrayList<>();
            List<String> field72TmpList = new ArrayList<>();
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PrvsInstgAgt1.FinInstnId");
            if (StringUtil.isEmpty(bicCode)) {
                bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PrvsInstgAgt.FinInstnId");
            }
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PrvsInstgAgt1.FinInstnId.Nm");
            if (StringUtil.isEmpty(name)) {
                name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.PrvsInstgAgt.FinInstnId.Nm");
            }
            if (StringUtil.isNotEmpty(bicCode)) {
                bicCode = Mx2MtConstants.INS + bicCode;
                field72TmpList.addAll(StringUtil.outStringList(bicCode, 35, "//"));
            } else if (StringUtil.isNotEmpty(name)) {
                name = Mx2MtConstants.INS + name;
                field72TmpList.addAll(StringUtil.outStringList(name, 35, "//"));
            }
            String instrForCdtrAgtCard = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.InstrForCdtrAgt.Cd");
            String instrForCdtrAgtInstrInf = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.InstrForCdtrAgt.InstrInf");
            if (instrForCdtrAgtCard == null && StringUtil.isNotEmpty(instrForCdtrAgtInstrInf)) {
                String cdtrAgtCode = Mx2MtConstants.ACC + instrForCdtrAgtInstrInf;
                field72TmpList.addAll(StringUtil.outStringList(cdtrAgtCode, 35, "//"));
            } else if (Instruction5Code.PHOB.value().equals(instrForCdtrAgtCard) && StringUtil.isNotEmpty(instrForCdtrAgtInstrInf)) {
                String cdtrAgtCode = Mx2MtConstants.PHONBEN + instrForCdtrAgtInstrInf;
                field72TmpList.addAll(StringUtil.outStringList(cdtrAgtCode, 35, "//"));
            } else if (Instruction5Code.TELB.value().equals(instrForCdtrAgtCard) && StringUtil.isNotEmpty(instrForCdtrAgtInstrInf)) {
                String cdtrAgtCode = Mx2MtConstants.PHONBEN + instrForCdtrAgtInstrInf;
                field72TmpList.addAll(StringUtil.outStringList(cdtrAgtCode, 35, "//"));
            }
            String instrForNxtAgtCard = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.InstrForNxtAgt.Cd");
            String instrForNxtAgtInstrInf = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.InstrForNxtAgt.InstrInf");
            if (StringUtil.isEmpty(instrForNxtAgtCard) && StringUtil.isNotEmpty(instrForNxtAgtInstrInf)) {
                String nextAgentCode = Mx2MtConstants.REC + instrForNxtAgtInstrInf;
                field72TmpList.addAll(StringUtil.outStringList(nextAgentCode, 35, "//"));
            } else if (Instruction4Code.TELA.value().equals(instrForNxtAgtCard) && StringUtil.isNotEmpty(instrForNxtAgtInstrInf)) {
                String nextAgentCode = Mx2MtConstants.TELE + instrForNxtAgtInstrInf;
                field72TmpList.addAll(StringUtil.outStringList(nextAgentCode, 35, "//"));
            }  else if (Instruction4Code.PHOA.value().equals(instrForNxtAgtCard) && StringUtil.isNotEmpty(instrForNxtAgtInstrInf)) {
                String field56Option = (String)map.get("field56Option");
                String nextAgentCode = "";
                if (StringUtil.isEmpty(field56Option)) {
                    nextAgentCode = Mx2MtConstants.PHON + instrForNxtAgtInstrInf;
                } else {
                    nextAgentCode = Mx2MtConstants.PHONIBK + instrForNxtAgtInstrInf;
                }
                field72TmpList.addAll(StringUtil.outStringList(nextAgentCode, 35, "//"));
            }
            for (int i=0; i<field72TmpList.size(); i++) {
                if (i == 6) break;
                if (i == 0) {
                    field72List.add(":72:" + field72TmpList.get(i));
                } else {
                    field72List.add(field72TmpList.get(i));
                }
            }
            map.put("field72List", field72List);

        } catch (Exception e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
