package com.brilliance.swift.mx2mt.mt202cov202.impl.cov;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 1，若<AnyBIC>节点有值，则取为B\50A，且BIC取此值
 * 2，若节点<IBAN>有值，则在此节点值前面加上“/”后，作为50A的账号；
 * 3，若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为B\50A的账号；否则直接取<Othr><Id>，则作为B\50A的账号。
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Dbtr><Id><OrgId><AnyBIC>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAcct><Id><Othr><SchmeNm><Cd>
 *      * 1，待补充？？？
 *  * 2，账号取值规则与B\50A相同
 *   * 1，若<Dbtr><Nm>有录入，则设置B\50 option为“K”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2，账号取值规则与B\50A相同
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Dbtr><Nm>
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Dbtr><PstlAdr><AdrLine>
 */
public class Field50AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field50AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        // TODO 50F取值待补充
        try {
            String field50BOption = "";
            String field50B = "";
            Document document = (Document) map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Dbtr.Id.OrgId");
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Dbtr.Nm");
            String account = getAccount();
            if (StringUtil.isNotEmpty(bicCode)) {
                field50BOption = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field50B = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field50B = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)) {
                field50BOption = "K";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field50B = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field50B = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Dbtr.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Dbtr.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field50B += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            }
            map.put("field50BOption", field50BOption);
            map.put("field50B", field50B);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAcct.Id.Othr.Id");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}

