package com.brilliance.swift.mx2mt.mt202cov202.impl.cov;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 1，若<BICFI>节点有值，则取为B\52A，且BIC取此值
 * 2，若节点<IBAN>有值，则在此节点值前面加上“/”后，作为52A的账号；
 * 3，若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为B\50A的账号；否则直接取<Othr><Id>，则作为B\52A的账号。
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgt><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgtAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgtAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgtAcct><Id><Othr><SchmeNm><Cd>
 *      * 1，若节点<Nm>有值，则设置B\52 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2，账号取值规则和节点与B\52A相同
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgt><FinInstnId><Nm>
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class FieldB52AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(FieldB52AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field52BOption = "";
            String field52B = "";
            Document document = (Document) map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAgt.FinInstnId");
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAgt.FinInstnId.Nm");
            String account = getAccount();
            if (StringUtil.isNotEmpty(bicCode)) {
                field52BOption = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field52B = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field52B = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)) {
                field52BOption = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field52B = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field52B = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAgt.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAgt.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field52B += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            }
            map.put("field52BOption", field52BOption);
            map.put("field52B", field52B);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAgtAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.DbtrAgtAcct.Id.Othr.Id");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}
