package com.brilliance.swift.mx2mt.mt202cov202.impl.cov;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 57A
 * 1,若节点<BICFI>有值，则设置B\57 option为“A”，同时BIC取此栏位；
 * 2,若节点<IBAN>有值，则作为57A的账号；
 * 3,若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为B\57A的账号；否则直接取<Othr><Id>，则作为B\57A的账号。
 * <CdtTrfTxInf>[1]<<UndrlygCstmrCdtTrf>CdtrAgt><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAgtAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAgtAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAgtAcct><Id><Othr><SchmeNm><Cd>
 * 57C
 * 如果<CdtrAgt><FinInstnId><BICFI>为空，且<CdtrAgt><FinInstnId><Nm>为空，但节点<IBAN>有值，或者节点<Othr><Id>有值，则设置B\57 option为“C”
 * 若节点<Id><Othr><Id>不为空，则取“//CH”+此节点作为账号，否则取节点<MmbId>作为账号：
 * 57d
 * 1，若节点<Nm>有值，则设置B\57 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 * 2，账号取值规则和节点与B\57A相同
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAgt><FinInstnId><Nm>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAgt><FinInstnId><PstlAdr><AdrLine>[1],<AdrLine>[2],<AdrLine>[3]
 */
public class FieldB57Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(FieldB57Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field57BOption = "";
            String field57B = "";
            Document document = (Document) map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgt.FinInstnId");
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgt.FinInstnId.Nm");
            String account = getAccount();
            if (StringUtil.isNotEmpty(bicCode)) {
                field57BOption = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field57B = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field57B = bicCode;
                }
            } else if (StringUtil.isNotEmpty(name)) {
                field57BOption = "D";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field57B = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field57B = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgt.FinInstnId.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgt.FinInstnId.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field57B += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            } else if (StringUtil.isNotEmpty(account)) {
                field57BOption = "C";
                String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgtAcct.Id.IBAN");
                String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgtAcct.Id.Othr.Id");
                String mmbId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgt.FinInstnId.ClrSysMmbId.MmbId");
                //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
                if (StringUtil.isNotEmpty(otherId)) {
                    account = "//CH" + otherId;
                } else if (StringUtil.isNotEmpty(mmbId)) {
                    account = "/" + mmbId;
                } else {
                    account = "/" + iban;
                }
                if (account.length() > 35) {
                    account = account.substring(0, 35);
                }
                field57B = account;
            }
            map.put("field57BOption", field57BOption);
            map.put("field57B", field57B);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgtAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAgtAcct.Id.Othr.Id");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}

