package com.brilliance.swift.mx2mt.mt202cov202.impl.cov;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 1,若节点<Nm>有值，则设置B\59 option为“”，且名称取自此节点，而地址需要合并截取？？？
 * 2,若节点<IBAN>有值，则在此节点的值前面加上“/”后，作为B\59账号的值；
 * 3,若节点<Othr><Id>有值，且节点<Othr><SchmeNm><Cd>有值，则在节点<Othr><Id>的值前面加上“//CH”后，作为B\59账号的值；
 * 4,若节点<Othr><Id>有值，且节点<Othr><SchmeNm><Cd>没值，则在节点<Othr><Id>的值前面加上“/”后，作为B\59账号的值；
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Cdtr><Nm>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Cdtr><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAcct><Id><Othr><SchmeNm><Cd>
 * 1,若节点<AnyBIC>有值，则设置B\59 option为“A”，同时BIC取此栏位；
 * 2，账号取值规则和节点与B\59相同
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Cdtr><Id><OrgId><AnyBIC>
 */
public class FieldB59Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(FieldB59Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            String field59BOption = "";
            String field59B = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Cdtr.Id.OrgId");
            String account = getAccount();
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Cdtr.Nm");
            if (StringUtil.isNotEmpty(name)){
                field59BOption = "NOLETTER";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field59B = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field59B = name;
                }
                int addressCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Cdtr.PstlAdr.AdrLine");
                if (addressCounts > 0) {
                    for (int i=0; i<addressCounts; i++) {
                        if (i == 3) break;
                        String address = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.Cdtr.PstlAdr.AdrLine("+i+")");
                        if (address.length() > 35) address = address.substring(0, 35);
                        field59B += Mx2MtConstants.NEW_LINE + address;
                    }
                }
            } else if (StringUtil.isNotEmpty(bicCode)) {
                field59BOption = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field59B = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field59B = bicCode;
                }
            }
            map.put("field59BOption", field59BOption);
            map.put("field59B", field59B);
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }

    private String getAccount() {
        try {
            String account = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAcct.Id.IBAN");
            String otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.UndrlygCstmrCdtTrf.CdtrAcct.Id.Othr.Id");
            //String schmeNameCd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "CdtTrfTxInf.DbtrAgtAcct.Id.SchmeNm.Cd");
            if (StringUtil.isNotEmpty(iban)) {
                account = "/" + iban;
            } else if (StringUtil.isNotEmpty(otherId)) {
                account = "//CH" + otherId;
            }
            if (StringUtil.isNotEmpty(account) && account.length() > 35) {
                account = account.substring(0, 35);
            }
            return account;
        } catch (Exception e) {
            throw new SwiftException(e.getMessage());
        }
    }
}


