package com.brilliance.swift.mx2mt.mt900910;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtCreator;
import com.brilliance.swift.mx2mt.Mx2MtFieldsGenerate;
import com.brilliance.swift.mx2mt.mt900910.impl.*;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import com.prowidesoftware.swift.model.mx.dic.CreditDebitCode;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class Mx2Mt900910Creator extends AbstractMx2MtCreator {

    private static final Logger logger = LoggerFactory.getLogger(Mx2Mt900910Creator.class);

    @Override
    public void preProcess() {
        Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
        Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
        map.put(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, parentElementMaps.get("GrpHdr"));
        map.put(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, parentElementMaps.get("Ntfctn"));
        map.put(Mx2MtConstants.MT_TEMPLATE_NAME, "swift_mt_900_910.vm");
    }

    @Override
    public void generateHead3() {
        String head3 = "";
        Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
        String uetr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.Refs.UETR");
        if (StringUtil.isNotEmpty(uetr)) {
            head3 += "{121:"+uetr+"}";
        }
        map.put("head3", head3);
    }

    @Override
    public List<Mx2MtFieldsGenerate> getFieldsGenerateList() {
        List<Mx2MtFieldsGenerate> fieldsGenerateList = new ArrayList<>();
        fieldsGenerateList.add(new Field13DGenerate());
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field21Generate());
        fieldsGenerateList.add(new Field25aGenerate());
        fieldsGenerateList.add(new Field32AGenerate());
        fieldsGenerateList.add(new Field50aGenerate());
        fieldsGenerateList.add(new Field52aGenerate());
        fieldsGenerateList.add(new Field56aGenerate());
        fieldsGenerateList.add(new Field72Generate());
        return fieldsGenerateList;
    }

    @Override
    public void postProcess() {
        String mtType = (String)map.get(Mx2MtConstants.MT_TYPE);
        if (Mx2MtConstants.MT_TYPE_910.equals(mtType)) {
            String field50a = (String)map.get("field50a");
            String field52a = (String)map.get("field52a");
            if (StringUtil.isEmpty(field50a) && StringUtil.isEmpty(field52a)) {
                throw new SwiftException("ERROR", "Either field 50a or field 52a must be present.");
            }
        }
    }
    @Override
    protected String getMtType() {
        try {
            String mtType = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String cdtDbtInd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.CdtDbtInd");
            //judge 900 or 910
            if (CreditDebitCode.DBIT.equals(CreditDebitCode.valueOf(cdtDbtInd))) {
                mtType = Mx2MtConstants.MT_TYPE_900;
            } else if (CreditDebitCode.CRDT.equals(CreditDebitCode.valueOf(cdtDbtInd))) {
                mtType = Mx2MtConstants.MT_TYPE_910;
            }
            map.put(Mx2MtConstants.MT_TYPE, mtType);
            return mtType;
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
