package com.brilliance.swift.mx2mt.mt900910.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.StringUtil;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 50A
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><Dbtr><Pty><Id><OrgId><AnyBIC>
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><DbtrAcct><Id><IBAN>
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><DbtrAcct><Id><Othr><Id>
 *
 * 50K
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><Dbtr><Pty><Nm>
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><Dbtr><Pty><PstlAdr><AdrLine>
 */
public class Field50aGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field50aGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        String mtType = (String)map.get(Mx2MtConstants.MT_TYPE);
        if (!Mx2MtConstants.MT_TYPE_910.equals(mtType)) {
            return;
        }
        try {
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String field50aOption = "";
            String field50a = "";
            String bicCode = getBicCode(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdPties.Dbtr.Id.OrgId");
            String name = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdPties.Dbtr.Nm");
            String account = getAccount();
            if(StringUtil.isNotEmpty(bicCode)){
                field50aOption = "A";
                if (StringUtil.isNotEmpty(account)) {
                    field50a = account + Mx2MtConstants.NEW_LINE + bicCode;
                } else {
                    field50a = bicCode;
                }
            }else if(StringUtil.isNotEmpty(name)){
                field50aOption = "K";
                if (name.length() > 35) name = name.substring(0, 35);
                if (StringUtil.isNotEmpty(account)) {
                    field50a = account + Mx2MtConstants.NEW_LINE + name;
                } else {
                    field50a = name;
                }
                int adrLineCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdPties.Dbtr.PstlAdr.AdrLine");
                if(adrLineCounts>0){
                    for (int i=0; i<adrLineCounts; i++) {
                        if (i == 3) {
                            break;
                        }
                        String adrLine = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdPties.Dbtr.PstlAdr.AdrLine("+i+")");
                        if (adrLine.length() > 35) adrLine = adrLine.substring(0, 35);
                        field50a += Mx2MtConstants.NEW_LINE + adrLine;
                    }

                }
            }
            if (StringUtil.isNotEmpty(field50aOption) && StringUtil.isNotEmpty(field50a)) {
                map.put("field50aOption", field50aOption);
                map.put("field50a", field50a);
            }
        } catch (Exception e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
    private String getAccount(){
        Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
        String account = "";
        String otherId = "";
        String iban = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdPties.DbtrAcct.Id.IBAN");
        if(StringUtil.isEmpty(iban)){
            otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdPties.DbtrAcct.Id.Othr.SchmeNm.Cd");
            if(StringUtil.isEmpty(otherId)){
                otherId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Ntfctn.Ntry.NtryDtls.TxDtls.RltdPties.DbtrAcct.Id.Othr.Id");
            }
        }
        if (StringUtil.isNotEmpty(iban)) {
            account = "/"+iban;
        } else if (StringUtil.isNotEmpty(otherId)) {
            account = "//CH"+otherId;
        }
        if (StringUtil.isNotEmpty(account) && account.length() > 35) {
            account = account.substring(0, 35);
        }
        return account;
    }
}
