package com.brilliance.swift.mx2mt.mt940950;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtCreator;
import com.brilliance.swift.mx2mt.Mx2MtFieldsGenerate;
import com.brilliance.swift.mx2mt.mt940950.impl.*;
import com.brilliance.swift.util.XmlUtil;
import org.apache.commons.lang.StringUtils;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class Mx2Mt940950Creator extends AbstractMx2MtCreator {

    private static final Logger logger = LoggerFactory.getLogger(Mx2Mt940950Creator.class);

    /**
     * Object MxCamt05300102
     * receiverBic
     * senderBic
     * field20
     * field25a
     * field28c
     * field60Option
     * field60F
     * field60A
     * field61List
     * field62Option
     * field62F
     * field62M
     * field64
     */
    @Override
    public void preProcess() {
        Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
        Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
        map.put(Mx2MtConstants.GRPHDR_PARENT_ELEMENT_NAME, parentElementMaps.get("GrpHdr"));
        map.put(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, parentElementMaps.get("Stmt"));
        map.put(Mx2MtConstants.MT_TEMPLATE_NAME, "swift_mt_940_950.vm");
    }
    @Override
    public List<Mx2MtFieldsGenerate> getFieldsGenerateList() {

        List<Mx2MtFieldsGenerate> fieldsGenerateList = new ArrayList<>();

        if (getMtMsg()){
            fieldsGenerateList.add(new Field21Generate());
            fieldsGenerateList.add(new Field65Generate());
            fieldsGenerateList.add(new Field86Generate());
        }
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field25aGenerate());
        fieldsGenerateList.add(new Field28CGenerate());
        fieldsGenerateList.add(new Field60Generate());
        fieldsGenerateList.add(new Field61ListGenerate());
        fieldsGenerateList.add(new Field62Generate());
        fieldsGenerateList.add(new Field64Generate());

        return fieldsGenerateList;
    }

    /**
     * 符合以下其中一个条件，则为940，否则为950报文
     * （1）<Stmt><Ntry>[j]数组中，节点<AddtlNtryInf>有值
     * （2）<Stmt><Bal>[i]数组中，节点<Tp><CdOrPrtry><Cd>值为“FWAV”
     * （3）节点<Stmt><AddtlStmtInf>有值
     * @return true:940, false:950
     */
    public boolean getMtMsg() {
        try {
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            String addtlNtryInf = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry.AddtlNtryInf");
            if(StringUtils.isNotEmpty(addtlNtryInf)){
                return true;
            }
            String cd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry.Bal.Tp.CdOrPrtry.Cd");
            if(StringUtils.isNotEmpty(cd)){
                return true;
            }
            String addtlStmtInf = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "stmt.AddtlStmtInf");
            if(StringUtils.isNotEmpty(addtlStmtInf)){
                return true;
            }
            return false;
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Mx2Mt950Creator.getMtMsg() error");
        }
    }
    @Override
    protected String getMtType() {
        if (getMtMsg()) {
            return Mx2MtConstants.MT_TYPE_940;
        } else {
            return Mx2MtConstants.MT_TYPE_950;
        }
    }

    /**
     * set field 21 = ""
     * set filed 65 = ""
     * set filed 86 = ""
     */
    @Override
    public void postProcess() {
        if (Mx2MtConstants.MT_TYPE_950.equals(getMtType())) {
            map.put("field21", "");
            map.put("field65", "");
            map.put("field86", "");
        }
    }
}
