package com.brilliance.swift.mx2mt.mt940950.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.NumberUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.mx.dic.BalanceType12Code;
import com.prowidesoftware.swift.model.mx.dic.CreditDebitCode;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigDecimal;

/**
 * <Stmt><Bal>[i]<Tp><CdOrPrtry><Cd>
 * <Stmt><Bal>[i]<CdtDbtInd>
 * <Stmt><Bal>[i]<Dt><Dt>
 * <Stmt><Bal>[i]<Amt Ccy=“XXX”>
 * i = 1
 */
public class Field60Generate extends AbstractMx2MtFieldsGenerate{

    private static final Logger logger = LoggerFactory.getLogger(Field60Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException{
        try {
            map.put("field60", "");
            String filed60Option = "";
            String crdr = "";
            String dt = "";
            String ccy = "";
            String amt = "";
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            int balCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal");
            if(balCounts>0){
                String cd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal.Tp.CdOrPrtry.Cd");
                if (BalanceType12Code.PRCD.value().equals(cd)) {
                    filed60Option = "F";
                } else if (BalanceType12Code.ITBD.value().equals(cd)) {
                    filed60Option = "M";
                } else {
                    throw new SwiftException("ERROR", "cannot find filed60Option.");
                }
                map.put("field60Option", filed60Option);
                String cdtDbtInd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal.CdtDbtInd");
                if (CreditDebitCode.CRDT.value().equals(cdtDbtInd)) {
                    crdr = Mx2MtConstants.DRCRMARK_C;
                } else if (CreditDebitCode.DBIT.value().equals(cdtDbtInd)) {
                    crdr = Mx2MtConstants.DRCRMARK_D;
                } else {
                    throw new SwiftException("ERROR", "cannot find filed60 D/C Mark.");
                }
                String dateStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal.Dt.Dt");
                if(StringUtil.isNotEmpty(dateStr)){
                    XMLGregorianCalendar XmlDt = DatatypeFactory.newInstance().newXMLGregorianCalendar(dateStr);
                    dt = DateUtil.format(XmlDt, "yyMMdd");
                }
                String amtStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal.Amt");
                String ccyStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal.Amt@Ccy");
                if (StringUtil.isNotEmpty(amtStr) && StringUtil.isNotEmpty(ccyStr)) {
                    amt = NumberUtil.formatAmt(new BigDecimal(amtStr), ccyStr);
                    ccy = ccyStr;
                }
                map.put("field60", crdr+dt+ccy+amt);
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
