package com.brilliance.swift.mx2mt.mt940950.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.NumberUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.mx.dic.CreditDebitCode;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * <Stmt><Ntry>[j]<ValDt><Dt>
 * <Stmt><Ntry>[j]<BookgDt><Dt>
 * <Stmt><Ntry>[j]<CdtDbtInd>
 * <Stmt><Ntry>[j]<RvslInd>
 * <Stmt><Ntry>[j]<Sts>
 * <Stmt><Ntry>[j]<Amt Ccy=“XXX”>
 * <Stmt><Ntry>[j]<BkTxCd><Prtry><Cd>
 * <Stmt><Ntry>[j]<NtryDtls><TxDtls><Refs><EndToEndId>
 * <Stmt><Ntry>[j]<AcctSvcrRef>
 * <Stmt><Ntry>[j]<NtryDtls><TxDtls><AddtlTxInf>
 */
public class Field61ListGenerate extends AbstractMx2MtFieldsGenerate{

    private static final Logger logger = LoggerFactory.getLogger(Field61ListGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException{
        try {
            String valueDateStr = "";
            String entryDateStr = "";
            String drCrMark = "";
            String amt = "";
            String ccy = "";
            List<String> filed61List = new ArrayList<>();
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            int ntryCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry");
            if(ntryCounts>0){
                for (int i = 0; i < ntryCounts; i++) {
                    String dt = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").ValDt.Dt");
                    String dt2 = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").BookgDt.Dt");
                    if(StringUtil.isNotEmpty(dt)){
                        XMLGregorianCalendar valueDateDt = DatatypeFactory.newInstance().newXMLGregorianCalendar(dt);
                        valueDateStr = DateUtil.format(valueDateDt, "yyMMdd");
                    }
                    if(StringUtil.isNotEmpty(dt2)){
                        XMLGregorianCalendar entryDateDt = DatatypeFactory.newInstance().newXMLGregorianCalendar(dt);
                        entryDateStr = DateUtil.format(entryDateDt, "yyMMdd");
                    }
                    String cdtDbtInd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").CdtDbtInd");
                    String rvslInd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").RvslInd");
                    if (CreditDebitCode.CRDT.value().equals(cdtDbtInd)) {
                        if (Boolean.TRUE.equals(Boolean.getBoolean(rvslInd))) {
                            drCrMark = Mx2MtConstants.DRCRMARK_RD;
                        } else {
                            drCrMark = Mx2MtConstants.DRCRMARK_C;
                        }
                    } else if (CreditDebitCode.DBIT.value().equals(cdtDbtInd)) {
                        if (Boolean.TRUE.equals(Boolean.getBoolean(rvslInd))) {
                            drCrMark = Mx2MtConstants.DRCRMARK_RC;
                        } else {
                            drCrMark = Mx2MtConstants.DRCRMARK_D;
                        }
                    } else {
                        throw new SwiftException("ERROR", "cannot find filed61 D/C Mark.");
                    }
                    String amtStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").Amt");
                    String ccyStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").Amt@Ccy");
                    if (StringUtil.isNotEmpty(amtStr) && StringUtil.isNotEmpty(ccyStr)) {
                        amt = NumberUtil.formatAmt(new BigDecimal(amtStr), ccyStr);
                        ccy = ccyStr;
                    }
                    String cd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").BkTxCd.Prtry.Cd");
                    String endToEndId = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").NtryDtls.TxDtls.Refs.EndToEndId");
                    String addtlTxInf = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").NtryDtls.TxDtls.AddtlTxInf");
                    String acctSvcrRef = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").AcctSvcrRef");
                    StringBuilder sb = new StringBuilder();
                    sb.append(valueDateStr)
                            .append(entryDateStr)
                            .append(drCrMark)
                            .append(ccy)
                            .append(amt)
                            .append(cd)
                            .append(endToEndId);
                    if (StringUtil.isNotEmpty(acctSvcrRef)) {
                        sb.append("//"+acctSvcrRef);
                    }
                    if (StringUtil.isNotEmpty(addtlTxInf)) {
                        sb.append(Mx2MtConstants.NEW_LINE+addtlTxInf);
                    }
                    String addtlNtryInf = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Ntry("+i+").AddtlNtryInf");
                    if(StringUtil.isNotEmpty(addtlNtryInf)){
                        sb.append(Mx2MtConstants.NEW_LINE).append(":86:").append(StringUtil.getStringByEnter(addtlNtryInf, 35, 6));
                    }
                    filed61List.add(sb.toString());
                }
            }
            if (filed61List.size() > 0) {
                map.put("field61List", filed61List);
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
