package com.brilliance.swift.mx2mt.mt940950.impl;

import com.brilliance.swift.constants.Mx2MtConstants;
import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2mt.AbstractMx2MtFieldsGenerate;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.NumberUtil;
import com.brilliance.swift.util.StringUtil;
import com.prowidesoftware.swift.model.mx.dic.BalanceType12Code;
import com.prowidesoftware.swift.model.mx.dic.CreditDebitCode;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigDecimal;

/**
 * <Stmt><Bal>[i]<Tp><CdOrPrtry><Cd>
 * <Stmt><Bal>[i]<CdtDbtInd>
 * <Stmt><Bal>[i]<Dt><Dt>
 * <Stmt><Bal>[i]<Amt Ccy=“XXX”>
 *  LOOP BAL
 *  IF CLAV == <Bal><Tp><CdOrPrtry><Cd> means exist 65
 */
public class Field65Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field65Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException{
        try {
            map.put("field65", "");
            Document document = (Document)map.get(Mx2MtConstants.MX_XML_DOCUMENT);
            int balCounts = getXmlNodeCounts(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal");
            if(balCounts>0){
                for (int i = 0; i < balCounts; i++) {
                    String cd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal("+i+").Tp.CdOrPrtry.Cd");
                    if(BalanceType12Code.FWAV.value().equals(cd)){
                        String cdtDbtInd = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal("+i+").CdtDbtInd");
                        String crdr = "";
                        if (CreditDebitCode.CRDT.value().equals(cdtDbtInd)) {
                            crdr = Mx2MtConstants.DRCRMARK_C;
                        } else if (CreditDebitCode.DBIT.value().equals(cdtDbtInd)) {
                            crdr = Mx2MtConstants.DRCRMARK_D;
                        } else {
                            throw new SwiftException("ERROR", "cannot find filed60 D/C Mark.");
                        }
                        String dt = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal("+i+").Dt.Dt");
                        String dateStr = "";
                        if (StringUtil.isNotEmpty(dt)) {
                            XMLGregorianCalendar dateDt = DatatypeFactory.newInstance().newXMLGregorianCalendar(dt);
                            dateStr = DateUtil.format(dateDt, "yyMMdd");
                        }
                        String amtStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal("+i+").Amt");
                        String ccyStr = getXmlNodeValue(Mx2MtConstants.MAINBODY_PARENT_ELEMENT_NAME, document, "Stmt.Bal("+i+").Amt@Ccy");
                        String amt = "";
                        String ccy = "";
                        if (StringUtil.isNotEmpty(amtStr) && StringUtil.isNotEmpty(ccyStr)) {
                            amt = NumberUtil.formatAmt(new BigDecimal(amtStr), ccyStr);
                            ccy = ccyStr;
                        }
                        map.put("field65", crdr+dateStr+ccy+amt);
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
}
